/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 905-2160 Terry-Fox                                                             *
 * Laval (Quebec)                                                                 *
 * Canada, H7T 3B7                                                                *
 *                                                                                *
 * support@codecpro.com                                                           *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *--------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h> 
#include <stdarg.h> 
#include "OS_depend_func.h"

//--------------------------------------------------------------------------------;
//  These are system or OS dependent functions. if these are completely 
//  different from standard functions called by codecLib eg. the embedded systems,
//  then, rewrite those functions for your owwn systems
//--------------------------------------------------------------------------------;

//#define PRINT_2_FILE
#ifdef PRINT_2_FILE
   FILE *logFile=0;
#endif


/* function print_msg ------------------------------------------- */
void print_msg(CHAR *fmt,va_list ap,CHAR* where)
{
    CHAR buf[500];

    vsprintf(buf,fmt,ap);
    if (where==0) {

#ifdef PRINT_2_FILE
        if (logFile == 0)
            logFile = fopen("c:/_debug/codecLib.log", "-a");
        fprintf(logFile, "%s",buf);
#else
        printf("%s",buf);
#endif
    }
    else{
        printf("codecLib_printf problem \n");
        exit(1);
 }

}/* end print_msg */


/*---------------------------*/
/*----- codecLib_printf -----*/
/*---------------------------*/
void codecLib_printf( CHAR *fmt, ...)
{
    va_list ap;

    va_start(ap,fmt);
    print_msg(fmt,ap,0);
    va_end(ap);
}/* end msg */


/*---------------------------*/
/*----- codecLib_malloc -----*/
/*---------------------------*/
void *codecLib_malloc(size_t n)
{
    void* memptr;
    memptr = malloc(n);    

   if (memptr)
	   codecLib_memset(memptr,0,n);

    return memptr;
}


/*---------------------------*/
/*----- codecLib_memcpy -----*/
/*---------------------------*/
void *codecLib_memcpy(void *ptr1, const void *ptr2, size_t n)
{
    void * rp;
    //
    // Use memmove to handle overlapping regions - memcpy gives error with ICC 64 bit Linux
    // with at least AMR. The exact instruction that gave the error was pin-pointed
    // in file sp_dec.c around line 5267 but we decided to use memmove instead of memcpy
    // widely to prevent any such error in the future.
    rp = memmove((void*)ptr1, (const void*)ptr2, (size_t)n);
    // rp = memcpy((void*)ptr1, (const void*)ptr2, (size_t)n);
    return rp;
}

// it is useful to remove the below functions only with shared libraries. Otherwise
// it defeats the purpose when delivering static libraries.
#if 1

// use OS functions, defined in OS_depend.h

/*---------------------------*/
/*----- codecLib_memmove-----*/
/*---------------------------*/
void *codecLib_memmove(void *ptr1, const void *ptr2, size_t n)
{
    void * rp;
    rp = memmove((void*)ptr1, (const void*)ptr2, (size_t)n);
    return rp;
}

/*---------------------------*/
/*----- codecLib_memset -----*/
/*---------------------------*/
void *codecLib_memset(void *ptr, int i, size_t n)
{
    void* rp;
    rp = memset(ptr, i, n);
    return rp;
}

/*---------------------------*/
/*----- codecLib_calloc -----*/
/*---------------------------*/
void * codecLib_calloc(size_t n, size_t t)
{
    return(calloc(n, t));
}

/*-------------------------*/
/*----- codecLib_free -----*/
/*-------------------------*/
void codecLib_free(void* ptr)
{
    free(ptr);
}

#endif

/*--------------------------*/
/*         end of file        */
/*--------------------------*/


