/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 905-2160 Terry-Fox                                                             *
 * Laval (Quebec)                                                                 *
 * Canada, H7T 3B7                                                                *
 *                                                                                *
 * support@codecpro.com                                                           *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *--------------------------------------------------------------------------------*/

/*=================================================================================
   EVS Codec 3GPP TS26.442 (see disclaimer.h for version number)
 =================================================================================*/

#ifndef MIME_H
#define MIME_H MIME_H

#include <stdio.h>
#include "../typedef.h"

/*
 * Header, data  and Magic Number values
  */

#define MIME_SYNC_GOOD_FRAME (unsigned short) 0x6B21
#define MIME_SYNC_BAD_FRAME  (unsigned short) 0x6B20
#define MIME_BIT0            (unsigned short) 0x007F
#define MIME_BIT1            (unsigned short) 0x0081
#define RTP_HEADER_PART1     (short)22               /* magic number by network simulator */


/*
 * ENUMS
 */

/* error enums */

typedef enum _MIME_ERROR
{
    MIME_NO_ERROR          = 0x0000,
    MIME_MEMORY_ERROR      = 0x0001,
    MIME_WRONG_PARAMS      = 0x0002,
    MIME_INIT_ERROR        = 0x0003,
    MIME_WRITE_ERROR       = 0x0004,
    MIME_READ_ERROR        = 0x0005,
    MIME_FILE_NOT_FOUND    = 0x0006,
    MIME_EOF               = 0x0007,
    MIME_NOT_IMPLEMENTED   = 0x0010,
    MIME_NOT_INITIALIZED   = 0x0100,
    MIME_UNKNOWN_ERROR     = 0x1000
} MIME_ERROR;

/*
 * Structures
 */

/* main handle */
struct __MIME;
typedef struct __MIME * MIME_HANDLE;

/*
 * Functions
 */

MIME_ERROR
MIME_Reader_Open(MIME_HANDLE* phMIME, FILE * filename);

MIME_ERROR
MIME_ReadVoipFrame_compact(MIME_HANDLE const hmime,
                           unsigned char * const serial,
                           short * const num_octets,
                           unsigned short * const rtpSequenceNumber,
                           unsigned int * const rtpTimeStamp,
                           unsigned int * const rcvTime_ms);


MIME_ERROR
MIME_Reader_Close(MIME_HANDLE* phMIME);

#endif /* MIME_H */
