/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 905-2160 Terry-Fox                                                             *
 * Laval (Quebec)                                                                 *
 * Canada, H7T 3B7                                                                *
 *                                                                                *
 * support@codecpro.com                                                           *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *--------------------------------------------------------------------------------*/

/*=================================================================================
     EVS Codec 3GPP TS26.442 (see disclaimer.h for version number)
 =================================================================================*/

#pragma once
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum _RTPDUMP_ERROR
{
  RTPDUMP_NO_ERROR          = 0x0000,
  RTPDUMP_MEMORY_ERROR      = 0x0001,
  RTPDUMP_WRONG_PARAMS      = 0x0002,
  RTPDUMP_INIT_ERROR        = 0x0003,
  RTPDUMP_WRITE_ERROR       = 0x0004,
  RTPDUMP_READ_ERROR        = 0x0005,
  RTPDUMP_FILE_NOT_FOUND    = 0x0006,
  RTPDUMP_NOT_IMPLEMENTED   = 0x0010,
  RTPDUMP_NOT_INITIALIZED   = 0x0100,
  RTPDUMP_READ_ENDOFFILE    = 0x0101,
  RTPDUMP_UNKNOWN_ERROR     = 0x1000
} RTPDUMP_ERROR;

typedef struct RTPDUMP *     RTPDUMP_HANDLE;
typedef struct RTPDUMP RTPDUMP;

typedef struct RTPDUMP_RTPPACKET
{
  unsigned char v_p_x_xx; /* version, padding, extension etc. */
  unsigned char payloadTypeId;
  unsigned short sequenceNumber;
  unsigned int timeStamp;
  unsigned int ssrc;
  char data[1500 + 12]; /* raw RTP packet */
  unsigned short headerSize;
  unsigned short payloadSize;
} RTPDUMP_RTPPACKET;

typedef struct {
    RTPDUMP_HANDLE rtpdump;
    int32_t  hf_only;
    RTPDUMP_RTPPACKET rtpPacket;
    uint32_t timeoffset_ms;
    uint16_t frameIndex;
    int32_t  frameFollowing;
} EVS_RTPDUMP_DEPACKER;

typedef enum {
    EVS_RTPDUMP_DEPACKER_NO_ERROR = 0,
    EVS_RTPDUMP_DEPACKER_EOF = -1,
    EVS_RTPDUMP_DEPACKER_RTPDUMP_ERROR = 1,
    EVS_RTPDUMP_DEPACKER_PAYLOAD_ERROR
} EVS_RTPDUMP_DEPACKER_ERROR;

EVS_RTPDUMP_DEPACKER_ERROR EVS_RTPDUMP_DEPACKER_open(EVS_RTPDUMP_DEPACKER* self, FILE* file, int32_t hf_only);

EVS_RTPDUMP_DEPACKER_ERROR EVS_RTPDUMP_DEPACKER_readNextFrame(
    EVS_RTPDUMP_DEPACKER* self,
    uint16_t* rtpSequenceNumber,
    uint32_t* rtpTimeStamp,
    uint32_t* rcvTime_ms,
    uint8_t*  toc,
    unsigned char** frame,
    uint16_t* frameSizeBytes);

void EVS_RTPDUMP_DEPACKER_close(EVS_RTPDUMP_DEPACKER* self);

RTPDUMP_ERROR
RTPDUMP_OpenForReading(RTPDUMP_HANDLE* phRTPDUMP, const char * filename);

RTPDUMP_ERROR
RTPDUMP_OpenWithFileToRead(RTPDUMP_HANDLE* phRTPDUMP, FILE *file);

RTPDUMP_ERROR
RTPDUMP_OpenForWriting(RTPDUMP_HANDLE* phRTPDUMP, const char * filename);

RTPDUMP_ERROR
RTPDUMP_ReadPacket(RTPDUMP_HANDLE hRTPDUMP,
                   RTPDUMP_RTPPACKET * packet,
                   uint32_t * timeoffset_ms);

RTPDUMP_ERROR
RTPDUMP_WritePacket(RTPDUMP_HANDLE hRTPDUMP,
                    const RTPDUMP_RTPPACKET * packet,
                    uint32_t timeoffset_ms);

void
RTPDUMP_Close(RTPDUMP_HANDLE* phRTPDUMP, short closeFile);

void
RTPDUMP_SetDefaultRtpPacketHeader(RTPDUMP_RTPPACKET * packet);

void
RTPDUMP_ParseRTPHeader(RTPDUMP_RTPPACKET * packet);

#ifdef __cplusplus
}
#endif
