/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 905-2160 Terry-Fox                                                             *
 * Laval (Quebec)                                                                 *
 * Canada, H7T 3B7                                                                *
 *                                                                                *
 * support@codecpro.com                                                           *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *--------------------------------------------------------------------------------*/

#ifndef EVS_IF_H
#define EVS_IF_H
#ifdef __cplusplus
extern "C" {
#endif

#include "typedef.h"

//
// Common encoder and decoder section starts here
//

extern const Word32 evs_major_version;
extern const Word32 evs_minor_version;
extern const Word32 evs_build_version;

#define AMRWB_MAGIC_NUMBER "#!AMR-WB\n"             /* defined in  RFC4867  */
#define EVS_MAGIC_NUMBER   "#!EVS_MC1.0\n"          /* defined in 26.445    */

#define MAX_BITS_PER_FRAME                    2560
#define L_FRAME48k                            960       /* Frame size in samples at 48kHz */
/*
    The format of the VoIP packet is as such:
    Octets 0  to 3      : System Time (MSB fist)
    Octets 4  to 5      : rtpPacketPayloadSize (MSB fist)
    Octets 6  to 9      : rtpPacketArrivalTime (MSB fist)
    Octets 10 to 11     : rtpSequenceNumber (MSB fist)
    Octets 12 to 15     : rtpTimeStamp (MSB fist)
    Octets 16 to (rtpPacketPayloadSize+16) : rtpPayload
*/
#define EVS_VOIP_HEADER_SIZE                  16

#define G192 0  // USED ONLY FOR CONFORMANCE TESTING
#define MIME 1
#define VOIP_RTPDUMP 3

#define FIXED_SID_RATE                        8         /* default DTX SID rate */
#define FEC_OFFSET                            3         /* default fec offset */

#define NB                                    0         /* Indicator of 4 kHz bandwidth */
#define WB                                    1         /* Indicator of 8 kHz bandwidth */
#define SWB                                   2         /* Indicator of 14 kHz bandwidth */
#define FB                                    3         /* Indicator of 20 kHz bandwidth */

/*----------------------------------------------------------------------------------*
 * Bitrates in bits per second
 *----------------------------------------------------------------------------------*/
#define ACELP_5k90                            5900      /* ACELP core layer at average bitrate of 5.90 kbps (used only in SC-VBR mode)      */
#define ACELP_6k60                            6600      /* ACELP core layer at 6.60  kbps (used only in AMR-WB IO mode)  */
#define ACELP_7k20                            7200      /* ACELP core layer at 7.20  kbps                                */
#define ACELP_8k00                            8000      /* ACELP core layer at 8     kbps                                */
#define ACELP_8k85                            8850      /* ACELP core layer at 8.85  kbps (used only in AMR-WB IO mode)  */
#define ACELP_9k60                            9600      /* ACELP core layer at 9.60  kbps                                */
#define ACELP_11k60                           11600     /* ACELP core layer at 11.60 kbps (used for SWB TBE)             */
#define ACELP_12k15                           12150     /* ACELP core layer at 12.15 kbps (used for WB TBE)              */
#define ACELP_12k65                           12650     /* ACELP core layer at 12.65 kbps (used only in AMR-WB IO mode)  */
#define ACELP_12k85                           12850     /* ACELP core layer at 12.85 kbps (used for WB BWE)              */
#define ACELP_13k20                           13200     /* ACELP core layer at 13.20 kbps                                */
#define ACELP_14k25                           14250     /* ACELP core layer at 14.25 kbps (used only in AMR-WB IO mode)  */
#define ACELP_14k80                           14800     /* ACELP core layer at 14.80 kbps (used only in core switching)  */
#define ACELP_15k85                           15850     /* ACELP core layer at 15.85 kbps (used only in AMR-WB IO mode)  */
#define ACELP_16k40                           16400     /* ACELP core layer at 16.40 kbps                                */
#define ACELP_18k25                           18250     /* ACELP core layer at 18.25 kbps (used only in AMR-WB IO mode)  */
#define ACELP_19k85                           19850     /* ACELP core layer at 19.85 kbps (used only in AMR-WB IO mode)  */
#define ACELP_22k60                           22600     /* ACELP core layer at 22.60 kbps (used only in core switching)  */
#define ACELP_23k05                           23050     /* ACELP core layer at 23.05 kbps (used only in AMR-WB IO mode)  */
#define ACELP_23k85                           23850     /* ACELP core layer at 23.85 kbps (used only in AMR-WB IO mode)  */
#define ACELP_24k40                           24400     /* ACELP core layer at 24.40 kbps                                */
#define ACELP_29k00                           29000     /* ACELP core layer at 29.00 kbps (used for FB + SWB TBE)        */
#define ACELP_29k20                           29200     /* ACELP core layer at 29.20 kbps (used for SWB TBE)             */
#define ACELP_30k20                           30200     /* ACELP core layer at 30.20 kbps (used for FB + SWB BWE)        */
#define ACELP_30k40                           30400     /* ACELP core layer at 30.40 kbps (used for SWB BWE)             */
#define ACELP_32k                             32000     /* ACELP core layer at 32    kbps                                */
#define ACELP_48k                             48000     /* ACELP core layer at 48    kbps                                */
#define ACELP_64k                             64000     /* ACELP core layer at 64    kbps                                */

#define HQ_16k40                              16400     /* HQ core at 16.4 kbps   */
#define HQ_13k20                              13200     /* HQ core at 13.2 kbps */
#define HQ_24k40                              24400     /* HQ core at 24.4 kbps */
#define HQ_32k                                32000     /* HQ core at 32 kbps */
#define HQ_48k                                48000     /* HQ core at 48 kbps */
#define HQ_64k                                64000     /* HQ core at 64 kbps */
#define HQ_96k                                96000     /* HQ core at 96 kbps */
#define HQ_128k                               128000    /* HQ core at 128 kbps */



//
// The ENCODER section starts here
//

// Data structure
// ==============
// Codec options structure is used to set the options for the encoder operation

typedef struct {
    Word32 input_Fs;            // Sampling rate of raw pcm audio signals in Hz {8000, 16000, 32000 or
                                // 48000}. (default = 16000)
    Word16 bitstreamformat;     // Bitstream format {G192 or MIME} (default = MIME)
    Word32 total_brate;         // Bitrate selection in kbps of the codec {ACELP_5k90, ACELP_6k60, ACELP_7k20,
                                // ACELP_8k85, ACELP_9k60, ACELP_11k60, ACELP_12k15, ACELP_12k65, ACELP_12k85,
                                // ACELP_8k00, ACELP_13k20, ACELP_14k25, ACELP_14k80, ACELP_15k85, ACELP_16k40,
                                // ACELP_18k25, ACELP_22k60, ACELP_23k05, ACELP_23k85, ACELP_24k40, ACELP_29k00,
                                // ACELP_29k20, ACELP_30k20, ACELP_30k40, ACELP_32k, ACELP_48k, ACELP_64k, HQ_16k40,
                                // HQ_13k20, HQ_24k40, HQ_32k, HQ_48k, HQ_64k, HQ_96k or HQ_128k} (encoder only)
    Word16 Opt_DTX_ON;          // DTX operation {disabled if 0, enabled otherwise}. (default = 0) (encoder only)
    Word16 interval_SID;        // SID update interval. Active when Opt_DTX_ON is enabled. {0 for variable CNG
                                // update interval; 3 to 100 for fixed CNG update interval} (default = 8)
    Word16 Opt_RF_ON;           // Redundant Frame operation or channel aware mode {disabled if 0, enabled
                                // otherwise}. (default = 0)
    Word16 rf_fec_offset;       // Redundant Frame - Frame Erasure Concealment - Offset.  Active when Opt_RF_ON is
                                // enabled. The difference in time units (20 ms) between the transmit time of the
                                // primary copy of a frame (n) and the transmit time of the redundant copy of that
                                // frame which is piggy backed onto a future frame (n + X) is called the FEC offset
                                // X. The optimal FEC offset is a value which maximizes the probability of
                                // availability of a partial redundant copy when there is a frame loss at the
                                // receiver. {0, 2, 3, 5 or 7} (default= 3)
    Word16 rf_fec_indicator;    // Redundant Frame - Frame Erasure Concealment - Indicator.  Active when Opt_RF_ON
                                // is enabled. This is the Frame Erasure Rate indicator. Set to LOW for FER rates
                                // <5% or to HIGH for FER>5%. This parameter controls the threshold used to
                                // determine whether a particular frame is critical or not. Such an adjustment of
                                // the criticality threshold is used to control the frequency of partial copy
                                // transmission. The high setting adjusts the criticality threshold to classify
                                // more frames as critical to transmit as compared to the low setting.
                                // {0 - LOW; 1 - HIGH} (default = 1 for HIGH)
    Word16 max_bwidth;          // Bandwidth limitation. {NB, WB, SWB or FB} (default = SWB)
   } evs_encoderOption;

Word32 E_IF_evs_queryBlockSize(
                Word32 *iEncSizeState,
		        Word32 *iEncSizeScratch
                );

Word32 E_IF_evs_init(
                void *hState,
	            void *hScratch,
                evs_encoderOption *stEncoderOption
                );

Word32 E_IF_evs_encode(
                void *hState,
                evs_encoderOption *stEncoderOption,
                Word16 *pSpeech, 
                UWord8 *pBitstream,
                Word32 *iOutputSize
               );

Word32 E_IF_evs_getInfo(
                evs_encoderOption *stEncoderOption,
                Word32 *iEncoderDelay,
                Word16 *iFrameSize
                );

Word32 E_IF_evs_close(
                void *state
                );

Word32 E_IF_evsPayload_packFrames(
                UWord8* bitstream,
                UWord32 cmr_fmtp,
                UWord32 cmr_value,
                Word32  hf_only,
                Word32  inputSize,
                Word32* outputSize
                );

// Encoder API return values
#define EVS_ENC_ERR_NONE                         0
#define EVS_ENC_ERR_GENERIC                     -1
#define EVS_ENC_ERR_SID_INTERVAL_OUT_OF_RANGE   -2
#define EVS_ENC_ERR_INVALID_BITRATE             -3
#define EVS_ENC_ERR_INVALID_SAMPLINGRATE        -4


//
// The DECODER section starts here
//

// Data structure
// ==============
// Codec options structure is used to set the options for the decoder operation

typedef struct {
    Word32 output_Fs;           // Sampling rate of raw pcm audio signals in Hz {8000, 16000, 32000 or
                                // 48000}. (default = 16000) (decoder)
    Word16 bitstreamformat;     // Bitstream format. (G192 used for conformance testing)
                                // When Opt_VOIP is disabled. {G192 or MIME};
                                // When Opt_VOIP is enabled. {G192, MIME or VOIP_RTPDUMP}
                                // (default = MIME)
    Word16 amrwb_rfc4867_flag;  // Use MIME rfc4867 ToC byte. Active when Opt_VOIP is disabled. {0 for MIME EVS, MIME
                                // rfc4867 otherwise} (default = 0)
    Word16 Opt_VOIP;            // VoIP and jitter buffer processing. {disabled if 0; enabled otherwise} (default = 0)
    Word16 jbmSafetyMargin;     // Jitter Buffer Safety Margin during RTP packet payload decoding. Active when
                                // Opt_VOIP is enabled. Allowed delay reserve in addition to network jitter to reduce
                                // late-loss. (default = 60 ms)
 } evs_decoderOption;

typedef struct {
    Word16 dec_delay;           // decoder compensation delay (to align decoded signal with original signal)
    Word32 bitrate;             // bitrate of bitstream.
    Word32 frame_Size;          // The number of octets in current frame.
    Word16 jitterBufferEmpty;   // 0 if jitter buffer is not empty;  jitter buffer is empty otherwise
    Word16 optimum_offset;      // Use optimum_offset and FEC_hi to determine the RTP EVS Channel Aware
    Word16 FEC_hi;              // Request (3GPP TS 26.114, table 1.b) to transmit.
} evs_streamInfo;


Word32 D_IF_evs_queryBlockSize(
                Word32 *iDecSizeState,
		        Word32 *iDecSizeScratch
                );

Word32 D_IF_evs_init(
                void *hState,
	            void *hScratch,
                evs_decoderOption *stDecoderOption
                );

Word32 D_IF_evs_decode(
                void *hState,
                evs_decoderOption *stDecoderOption,
                UWord8 *pbitstream,
                Word16 *pSynthSpeech,
                Word32 *iOutputSize,
                Word16 iBFI
                );

Word32 D_IF_evs_getFrameInfo(
                void *hState,
                evs_decoderOption *stDecoderOption,
                UWord8 *pBitstreamHeader,
                evs_streamInfo *stStreamInfo
                );

Word32 D_IF_evs_close(
                void *state
                );

Word32 D_IF_evsPayload_unpackFrame(
                Word32 hf_only,
                const UWord8* payload,
                UWord16  payloadSizeBytes,
                UWord16  frameIndex,
                Word32*  frameFollowing,
                UWord8*  toc_byte,
                UWord8** framePtr,
                UWord16* frameSizeBytes
                );

// Decoder API return values
#define EVS_DEC_FOUND_FIRST_GOOD_FRAME       1  // for testing purposes only (G.192)
#define EVS_DEC_ERR_NONE                     0
#define EVS_DEC_ERR_GENERIC                 -1
#define EVS_DEC_ERR_FAU                     -2  // JBM error in feeding access unit
#define EVS_DEC_ERR_GS                      -3  // JBM error in getting samples
#define EVS_DEC_ERR_INVALID_SAMPLINGRATE    -4
#define EVS_DEC_ERR_BAD_HEADER              -5

#ifdef __cplusplus
}
#endif

#endif /* EVS_H */
