/*--------------------------------------------------------------------------------*
 *                                                                                *
 * CodecPro Incorporated                                                          *
 * 905-2160 Terry-Fox                                                             *
 * Laval (Quebec)                                                                 *
 * Canada, H7T 3B7                                                                *
 *                                                                                *
 * support@codecpro.com                                                           *
 * www.codecpro.com                                                               *
 *                                                                                *
 * This material is subject to continuous developments and improvements. All      *
 * warranties implied or expressed, including but not limited to, implied         *
 * warranties of merchantability, fitness for purpose, condition of title,        *
 * noninfringement, are excluded. In no event shall CodecPro Incorporated and its *
 * suppliers be liable for any special, indirect or consequential damages or any  *
 * damages whatsoever arising out of or in connection with the use of this        *
 * information. The foregoing disclaimer shall apply to the maximum extent        *
 * permitted by applicable law, even if any remedy fails its essential purpose.   *
 *--------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wavefiletools.h"

static void ExtractFormat(CHAR *FormatString, Word16 *ReadFlag, Word16 *AppendFlag);
/*____________________________________________________________________________
 |
 |  FUNCTION NAME: Wave_Fopen
 |____________________________________________________________________________
*/

FILE *Wave_Fopen(CHAR *Filename, CHAR *Mode, Word16 *NumOfChannels, Word32 *SamplingRate, Word16 *BitsPerSample, Word32 *DataSize)
{
    FILE *FilePtr, *FilePtrAppend=NULL;
    Word16 ReadFlag;
    Word16 AppendFlag;
    CHAR   WaveTag[4];
    Word32 PCM_Size = 0L;
    Word32 FileSize = 0L;		/* Actual size will be replaces in "Wave_Fclose" -procedure */
    Word32 FormatSize = 16L;		
    Word32 FormatTag = 1;		/* Format Tag (PCM=1) */
    Word16 BytesPerSample;
    Word32 BytesPerSecond;
    Word32 longtmp,fmt_ckSizeOffset;
    Word16 shorttmp;
    Word32 sRate;

    ExtractFormat(Mode, &ReadFlag, &AppendFlag);
    if(!ReadFlag){
        if((FilePtr=fopen(Filename, Mode)) == NULL){
            return FilePtr;
        }
        if(!AppendFlag){
            if(*BitsPerSample != 8 && *BitsPerSample != 16){
                printf("\n\n");
                printf("\n ERROR: Only values 16 or 8 allowed for parameter 'BitsPerSample' in function Wave_Fopen");
                printf("\n        [value: %d received]", *BitsPerSample);
                printf("\n\n");
                exit(-1);
            }
            BytesPerSample = (Word16)(*BitsPerSample /8 * *NumOfChannels);
            BytesPerSecond = *SamplingRate * BytesPerSample;
            /* RIFF chunk - 12 bytes */	
            fwrite("RIFF", 1, 4, FilePtr);
            fwrite(&FileSize, 4, 1, FilePtr);
            fwrite("WAVE", 1, 4, FilePtr);
            /* FORMAT chunk - 24 bytes */
            fwrite("fmt ", 1, 4, FilePtr);
            fwrite(&FormatSize, 4, 1, FilePtr);
            fwrite(&FormatTag,  2, 1, FilePtr);
            fwrite(NumOfChannels,  2, 1, FilePtr);
            sRate = (*SamplingRate);
            fwrite(&sRate,  4, 1, FilePtr);
            fwrite(&BytesPerSecond, 4, 1, FilePtr);
            fwrite(&BytesPerSample,  2, 1, FilePtr);
            fwrite(BitsPerSample,  2, 1, FilePtr);
            /* DATA chunk - Data length + 8 bytes */
            fwrite("data",  1, 4, FilePtr);
            fwrite(&PCM_Size,  4, 1, FilePtr);
        }
        return FilePtr;
    }
    else{
        if((FilePtr=fopen(Filename, Mode)) == NULL){
            return FilePtr;
        }
        if(AppendFlag){
            FilePtrAppend = FilePtr;
        }
        fseek(FilePtr, 0, SEEK_SET);
        /* RIFF chunk - 12 bytes */
        if (fread(&WaveTag, 4, 1, FilePtr) != 4) return NULL;
        if(strncmp("RIFF", WaveTag, 4) != 0){
            return NULL;
        }
        if (fread(&longtmp, 4, 1, FilePtr) != 4) return NULL;
        if (fread(&WaveTag, 4, 1, FilePtr) != 4) return NULL;
        if(strncmp("WAVE", WaveTag, 4) != 0){
            return NULL;
        }
        /* FORMAT chunk - 24 bytes */
        if (fread(&WaveTag, 4, 1, FilePtr) != 4) return NULL;
        if(strncmp("fmt ", WaveTag, 4) != 0){
            while (strncmp("fmt ", WaveTag, 4) != 0){
                if (fread(&longtmp, 4, 1, FilePtr) != 4) return NULL; // size of non 'fmt ' chunck
                fseek(FilePtr, longtmp, SEEK_CUR);// positionning after chunck data
                if (fread(&WaveTag, 4, 1, FilePtr) != 4) return NULL;
            }
            if (feof(FilePtr)) return NULL; // quit if it's END OF FILE
        }
        if (fread(&longtmp, 4, 1, FilePtr) != 4) return NULL; // size of the structure format	
        fmt_ckSizeOffset = longtmp - 16;// could be 16,22 or 40
        if (fread(&shorttmp, 2, 1, FilePtr) != 2) return NULL;
        if(shorttmp != 0x0001){      // check if it's WAVE_FORMAT_PCM 
            return NULL;
        }
        if (fread(NumOfChannels,  2, 1, FilePtr) != 2) return NULL;
        if (fread(SamplingRate,  4, 1, FilePtr) != 4) return NULL;
        if (fread(&longtmp, 4, 1, FilePtr) != 4) return NULL;
        if (fread(&BytesPerSample,  2, 1, FilePtr) != 2) return NULL;
        if (fread(BitsPerSample,  2, 1, FilePtr) != 2) return NULL;

        if(fmt_ckSizeOffset){ 
            fseek(FilePtr, fmt_ckSizeOffset, SEEK_CUR);// positionning to fact if included
        }

        /* optionnal Chunks reading  until data chunk is read */
        while ((strncmp("data", WaveTag, 4) != 0)&& (!feof(FilePtr)))
        {
            if (fread(&WaveTag, 4, 1, FilePtr) != 4) return NULL;     // chunk id 
            if(strncmp("data", WaveTag, 4) == 0)
                break;
            if (fread(&longtmp,  4, 1, FilePtr) != 4) return NULL;    // chunk size of the optionnal structures
            fseek(FilePtr, longtmp, SEEK_CUR);// optionnal field upon fact_ckSize
        }

        if(strncmp("data", WaveTag, 4) != 0)   // if data chunk never found then file is not a good wave pcm
            return NULL;

        //Read data size to return by the function
        if (fread(DataSize,  4, 1, FilePtr) != 4) return NULL;
        *DataSize = *DataSize / BytesPerSample;
        if(AppendFlag) return FilePtrAppend;
        return FilePtr;
    }
}
/*____________________________________________________________________________
 |
 |  FUNCTION NAME: Wave_Fclose
 |____________________________________________________________________________
*/
Word32 Wave_Fclose(FILE* FilePtr, Word16 BitsPerSample)
{
    Word32 DataLen;
    Word32 FileLen;
    Word32 StartPos;
    Word32 EndPos;

    if (BitsPerSample != 8 && BitsPerSample != 16) {
        printf("\n\n");
        printf("\n ERROR: Only values 16 or 8 allowed for parameter 'BitsPerSample' in function Wave_Fclose");
        printf("\n\n");
        return -1;
    }
    EndPos = ftell(FilePtr);
    fseek(FilePtr, 0, SEEK_SET);
    StartPos = ftell(FilePtr);
    FileLen = ((EndPos - StartPos) / (BitsPerSample / 16)) - 8;
    DataLen = FileLen - 44 + 8;
    fseek(FilePtr, 4, SEEK_SET);
    fwrite(&FileLen, 1, 4, FilePtr);
    fseek(FilePtr, 40, SEEK_SET);
    fwrite(&DataLen, 1, 4, FilePtr);
    fclose(FilePtr);

    return 0;
}

void CheckIfWave_Fclose(FILE* FilePtr)
{

    Word32 DataLen;
    Word32 FileLen;
    Word32 StartPos;
    Word32 EndPos;
    UWord8 wavSignature[16];

    EndPos = ftell(FilePtr);
    fseek(FilePtr, 0, SEEK_SET);
    if (16 == fread(wavSignature, 1, 16, FilePtr)) {
        if ((strncmp(wavSignature, "RIFF",4) == 0) && (strncmp(&wavSignature[8], "WAVEfmt ", 8) == 0)) {

            fseek(FilePtr, 0, SEEK_SET);
            StartPos = ftell(FilePtr);
            FileLen = (EndPos - StartPos) - 8;
            DataLen = FileLen - 44 + 8;
            fseek(FilePtr, 4, SEEK_SET);
            fwrite(&FileLen, 1, 4, FilePtr);
            fseek(FilePtr, 40, SEEK_SET);
            fwrite(&DataLen, 1, 4, FilePtr);
            fclose(FilePtr);

            return;
        }
    }


    fclose(FilePtr);

}

/*____________________________________________________________________________
 |
 |  FUNCTION NAME: ExtractFormat
 |____________________________________________________________________________
*/
static void ExtractFormat(CHAR *FormatString, Word16 *ReadFlag, Word16 *AppendFlag)
{
    *ReadFlag = 0;
    *AppendFlag = 0;
    if(strchr(FormatString, (int)'a') != NULL) *AppendFlag = 1;
    if(strchr(FormatString, (int)'A') != NULL) *AppendFlag = 1;
    if(strchr(FormatString, (int)'r') != NULL) *ReadFlag = 1;
    if(strchr(FormatString, (int)'R') != NULL) *ReadFlag = 1;
}
